@@ -1715,4 +1715,52 @@ def insert_first_row_df(df, row):
     '''
     df.loc[-1] = row
     df.index = df.index + 1
-    return df.sort_index()
\ No newline at end of file
+    return df.sort_index()
+
+def run_cppi(risky_r = None, safe_rate = 0., m = 3, start = 100, floor = 0.8, drawdown = None):
+    safe_r = pd.DataFrame().reindex_like(risky_r)
+    safe_r[:] = safe_rate
+    start = 1000
+    floor = 0.8
+
+    dates = aggregated_df.index
+    n_steps = len(dates)
+    account_value = start
+    floor_value = start * floor
+    m = 3.
+    peak = start
+
+    account_history = pd.DataFrame().reindex_like(risky_r)
+    cushion_history = pd.DataFrame().reindex_like(risky_r)
+    risky_w_history = pd.DataFrame().reindex_like(risky_r)
+
+    for step in range(0, n_steps):
+        if drawdown is not None :
+            peak = np.maximum(account_value, peak)
+            floor_value = peak * (1-drawdown)
+        cushion = (account_value - floor_value) / account_value
+        risky_w = m * cushion
+        risky_w = np.minimum(risky_w, 1.)
+        risky_w = np.maximum(risky_w, 0.)
+        safe_w = 1 - risky_w
+        risky_alloc = account_value * risky_w
+        safe_alloc = account_value * safe_w
+        ## update the account value
+        account_value = risky_alloc * (1 + risky_r.iloc[step]) + safe_alloc * (1 + safe_r.iloc[step])
+        ## save the values
+        cushion_history.iloc[step] = cushion
+        risky_w_history.iloc[step] = risky_w
+        account_history.iloc[step] = account_value
+
+    risky_wealth = start * (1 + risky_r).cumprod()
+    backtest_result = {
+        'Wealth':account_history,
+        'Risky Wealth': risky_wealth,
+        'Risk Budget': cushion_history,
+        'Risky Allocation': risky_w_history,
+        'm': m,
+        'start' : start,
+        'floor' : floor,
+        'risky_r' : risky_r,
+        'safe_r' : safe_r}
+    return backtest_result
\ No newline at end of file
